
/**
 * @file oh_main.c
 * @version $Id$
 *
 * The source file that has initial starting point of the share part.
 */

#include "oh_port.h"
#include "oh_main.h"
#include "oh_fs.h"
#include "oh_args.h"
#include "oh_env.h"
#include "oh_envnm.h"
#include "oh_fsutl.h"
#include "oh_fs.h"
#include "oh_debug.h"

#ifdef OH_TESTKIT
#include "oh_tstkt.h"
#endif /* OH_TESTKIT */

#define _OH_EMPTY
#define _OH_DECLARE_PROJECT_WRAP(prj) \
  extern oh_int oh_##prj##_main(oh_int argc, oh_char **argv)
#define _OH_DECLARE_PROJECT(prj) _OH_DECLARE_PROJECT_WRAP(prj)
#define _OH_CALL_PROJECT_WRAP(prj, argc, argv) oh_##prj##_main(argc, argv)
#define _OH_CALL_PROJECT(prj, argc, argv) \
  _OH_CALL_PROJECT_WRAP(prj, argc, argv)

#define _OH_DEF2STR(def) #def

_OH_DECLARE_PROJECT(OH_PROJECT);

static oh_int _oh_bt_initOpts(oh_int *argcp, oh_char **argv);

/**
 * Initializes home directory and cfg file name.
 *
 * If there is no -h ( home directory ) command line argument, then it tries to
 * get home directory from the environment variable. The name of the
 * environment variable is given by calling oh_en_getEnvName(). If the home
 * environment variable is not available, then it uses the current directory
 * for the home directory. If the home directory is not real directory or not
 * accessible, then it uses the current directory for the home directory.
 *
 * If there is no -c ( cfg file name ) command line argument, then it tries to
 * get cfg file name from home directory. The default cfg file name is given by
 * calling oh_en_getEnvName().
 *
 * The parsed directory and file name are changed into the unix file format by
 * calling oh_fu_dos2unix(). If the name is already in unix file format, then
 * oh_fu_dos2unix() does nothing.
 *
 * argcp points to the length of argv array before calling, argcp points to the
 * length of argv array that is not parsed yet after returning. Parsed
 * arguments are removed from argv array.
 *
 * Parsed arguments are set globally by calling oh_as_setGlobalOpts().
 *
 * @param argcp The pointer to the length of argv array.
 * @param argv The array of the char strings of the arguments.
 * @return The exit code, 0 if it is ok, not 0 otherwise.
 */
static oh_int _oh_bt_initOpts(oh_int *argcp, oh_char **argv)
{
  oh_as_opts_t *retopts, *homeopt, *cfgopt;
  oh_char path[OH_FS_MAXPATHLEN];
  const oh_char *envname;

  retopts = oh_as_parseArgs("-h?:-c?:", argv, argcp);
  if (retopts == OH_NULL)
    goto cleanupret;
  /* if there is no -h argument */
  if ((homeopt = oh_as_findOpt(retopts, "-h")) == OH_NULL)
  {
    /* if there is home environment variable */
    envname = oh_en_getEnvName(OH_EN_HOME);
    if (oh_ev_getEnv(envname, path, OH_FS_MAXPATHLEN) == 0)
    {
      homeopt = oh_as_newOpt(2, "-h", path);
      if (homeopt == OH_NULL)
        goto cleanupret;
      oh_as_addOpt(retopts, homeopt);
    }
    else /* if there is no home environment variable */
    {
      homeopt = oh_as_newOpt(2, "-h", ".");
      if (homeopt == OH_NULL)
        goto cleanupret;
      oh_as_addOpt(retopts, homeopt);
    }
  }

  OH_ASSERT(homeopt != OH_NULL);

  {
    oh_int fd;
    oh_fs_stat_t stat;
    oh_fu_dos2unix(homeopt->opts[1]);
    fd = oh_fs_open(homeopt->opts[1], OH_FS_O_RDONLY);
    /* check if the home directory is accessible directory */
    if ((fd < 0) ||
      (oh_fs_fstat(fd, &stat) != 0) ||
      (!OH_FS_S_ISDIR(stat.st_mode)))
    {
      homeopt = oh_as_newOpt(2, "-h", ".");
      if (homeopt == OH_NULL)
      {
        oh_fs_close(fd);
        goto cleanupret;
      }
      oh_as_addOpt(retopts, homeopt);
    }
    oh_fs_close(fd);
  }

  cfgopt = oh_as_findOpt(retopts, "-c");
  /* if there is no -c argument */
  if (cfgopt == OH_NULL)
  {
    envname = oh_en_getEnvName(OH_EN_CFGNAME);
    oh_c_snprintf(path, OH_FS_MAXPATHLEN, "%s/%s", homeopt->opts[1], envname);
    cfgopt = oh_as_newOpt(2, "-c", path);
    if (cfgopt == OH_NULL)
      goto cleanupret;
    oh_as_addOpt(retopts, cfgopt);
  }
  oh_fu_dos2unix(cfgopt->opts[1]);

  oh_as_setGlobalOpts(retopts);
  return 0;

cleanupret:
  oh_as_freeOpts(retopts);
  return -1;
}

/**
 * Starts to boot and returns the exit code.
 *
 * It is called after platform dependent initialization. It is the share part
 * entry point. The meaning of arguments and return value are same with ANSI C
 * main function.
 *
 * If OH_TESTKIT is defined, then it calls oh_tk_main. After returning from
 * oh_tk_main, it calls the project dependent entry function that is defined
 * by OH_PROJECT.
 *
 * @param argc The length of argv array.
 * @param argv The array of the char strings of the arguments.
 * @return The exit code, 0 if it is ok, not 0 otherwise.
 */
oh_int oh_bt_main(oh_int argc, oh_char **argv)
{
  oh_int ret = -1;
  oh_as_opts_t *global_opts;

  oh_db_log((OL_DBG, OD_BOOT, "oh_bt_main begin"));

#ifdef OH_DEBUG
  {
    oh_int i;
    for (i = 0; i < argc; i++)
    {
      oh_db_log((OL_DBG, OD_BOOT, "oh_bt_main argv[%d] = #%s#", i, argv[i]));
    }
  }
#endif /* OH_DEBUG */

  if (_oh_bt_initOpts(&argc, argv) != 0)
    goto cleanupret;
  oh_db_log((OL_DBG, OD_BOOT, "#%s -h %s -c %s#", argv[0],
    oh_as_findOpt(oh_as_getGlobalOpts(), "-h")->opts[1],
    oh_as_findOpt(oh_as_getGlobalOpts(), "-c")->opts[1]));

#ifdef OH_TESTKIT
  ret = oh_tk_main(argc, argv);
#endif /* OH_TESTKIT */

  if (ret < 0) /* if oh_tk_main can't parse arguments */
    ret = _OH_CALL_PROJECT(OH_PROJECT, argc, argv);

cleanupret:
  global_opts = oh_as_getGlobalOpts();
  oh_as_freeOpts(global_opts);
  oh_as_setGlobalOpts(OH_NULL);

  oh_db_log((OL_DBG, OD_BOOT, "oh_bt_main end with exit code %d", ret));

  return ret;
}
