
/**
 * @file oh_args.c
 * @version $Id$
 *
 * The source file that has the command line argument parsing functions.
 */

#include "oh_port.h"
#include "oh_args.h"
#include "oh_fs.h"
#include "oh_debug.h"

static oh_as_opts_t *_oh_as_globalOpts;

static const oh_char *_oh_as_skipWhiteSpaces(const oh_char *buf);
static const oh_char *_oh_as_moveToWhiteSpace(const oh_char *buf);

static const oh_char *_oh_as_skipWhiteSpaces(const oh_char *buf)
{
  OH_ASSERT(buf != OH_NULL);

  for (; oh_c_isspace((oh_uchar)*buf); buf++)
    /* nothing */;

  return buf;
}

static const oh_char *_oh_as_moveToWhiteSpace(const oh_char *buf)
{
  OH_ASSERT(buf != OH_NULL);

  for (; *buf != '\0' && !oh_c_isspace((oh_uchar)*buf); buf++)
    /* nothing */;

  return buf;
}

/**
 * Parses the raw command line arguments string and returns the array of the
 * char strings.
 *
 * It parses the "rawargs" char string and returns the array of the char
 * strings. And it writes the length of the returned array to the memory that
 * "argcp" points to.
 *
 * The returned pointer is dynamically allocated memory, therefore it must be
 * deallocated by calling the oh_as_freeArgs() function.
 *
 * If there is an error, then returns OH_NULL.
 *
 * @param rawargs The raw command line arguments that will be parsed.
 * @param argcp The int pointer where the length of the returned array will be
 * written.
 * @return The array of the char strings that is the parsed raw command line
 * arguments if successful, OH_NULL otherwise.
 */
oh_char **oh_as_parseRawArgs(const oh_char *rawargs, oh_int *argcp)
{
  const oh_char *braw, *eraw;
  oh_char **argv, **argvptr, *argvstr;
  oh_long bufsize;
  oh_int argc;

  OH_ASSERT(rawargs != OH_NULL);
  OH_ASSERT(argcp != OH_NULL);

  /* calculate the required buffer size */
  for (eraw = rawargs, bufsize = 0, argc = 0;;)
  {
    braw = _oh_as_skipWhiteSpaces(eraw);
    eraw = _oh_as_moveToWhiteSpace(braw);
    if (braw == eraw)
      break;
    argc++;
    if (*braw == '"') /* quoted argument */
    {
      braw++;
      eraw = oh_c_strchr(braw, '"');
      if (eraw == OH_NULL)
        return OH_NULL;
    }
    /* 1 is for the null char. */
    bufsize += eraw - braw + 1 + sizeof(oh_char *);
    if (argc < 0 || bufsize < 0) /* overflow */
      return OH_NULL;
    if (*eraw == '"') /* if quoted argument, move to white space char */
      eraw++;
  }
  /* for the null pointer */
  bufsize += sizeof(oh_char *);

  argv = (oh_char **)oh_c_malloc(bufsize);
  if (argv == OH_NULL)
    return OH_NULL;

  argvptr = argv;
  /* 1 is for the null pointer */
  argvstr = (oh_char *)(argv + argc + 1);

  /* fill in the buffer */
  for (eraw = rawargs;;)
  {
    oh_ulong len;

    braw = _oh_as_skipWhiteSpaces(eraw);
    eraw = _oh_as_moveToWhiteSpace(braw);
    if (braw == eraw)
      break;
    if (*braw == '"') /* quoted argument */
    {
      braw++;
      eraw = oh_c_strchr(braw, '"');
    }

    len = (oh_ulong)(eraw - braw);
    oh_c_memcpy(argvstr, braw, len);
    argvstr[len] = '\0';
    *argvptr = argvstr;
    argvptr++;
    /* 1 is for the null char */
    argvstr = argvstr + len + 1;

    if (*eraw == '"') /* if quoted argument, move to white space char */
      eraw++;
  }
  *argvptr = OH_NULL;
  *argcp = argc;

  return argv;
}

/**
 * Deallocates the returned buffer from the oh_as_parseRawArgs() function.
 *
 * The oh_as_parseRawArgs() function returns the dynamically allocated memory
 * and that must be deallocated by calling the oh_as_freeArgs() function.
 *
 * @param args The array of the char strings that will be deallocated.
 * @return none.
 */
oh_void oh_as_freeArgs(oh_char **args)
{
  oh_c_free(args);
}

/**
 * Parses the array of the char strings with the specified format,
 * "argsformat".
 *
 * The returned pointer is dynamically allocated memory, therefore it must be
 * deallocated by calling the oh_as_freeOpts() function. The char string in the
 * "argv" array can be changed after returning from it. Because it copys the
 * char strings from "argv" array.
 *
 * The parsed arguments are removed from "argv". The "argcp" points to the
 * memory that has the length of the "argv" before calling. After the arguments
 * are removed from the "argv", the memory that the "argcp" points to are
 * updated also.
 *
 * ':' is the argument format separator. '.' means no argument. '?' means the
 * one argument. '*' means the multiple arguments that is separated with ','.
 *
 * For example, if "-h?:-p.:-q*" is argsformat and argv is { "prog", "-h",
 * "/c/home/ohhara", "-p", "-q", "fs,clib,args" }, it will be parsed correctly.
 *
 * If there are same arguments, then the latter one override the others.
 *
 * If there is an error, then returns OH_NULL. And unexpected char string is
 * removed from "argv" and unexpected data is written to the memory "argcp"
 * points to.
 *
 * @param argsformat The parsing format.
 * @param argv The array of the char string that will be parsed.
 * @param argcp The pointer of the length of the argv.
 * @return The parsed arguments if successful, OH_NULL otherwise.
 */
oh_as_opts_t *oh_as_parseArgs(const oh_char *argsformat, oh_char **argv,
  oh_int *argcp)
{
  oh_ulong argstrlen, optstrlen;
  oh_as_opts_t *opts, *tailopts, *newopts;
  const oh_char *bstr, *estr;
  oh_int i, j, k;

  OH_ASSERT(argsformat != OH_NULL);
  OH_ASSERT(argv != OH_NULL);
  OH_ASSERT(argv[0] != OH_NULL);
  argstrlen = oh_c_strlen(argv[0]);
  OH_ASSERT(argstrlen != 0);
  /* the first argument should not start with '-' */
  OH_ASSERT(argv[0][0] != '-');

  /* the first argument setup */
  opts = (oh_as_opts_t *)oh_c_calloc(1, sizeof(oh_as_opts_t));
  if (opts == OH_NULL)
    return OH_NULL;
  opts->opts = (oh_char **)oh_c_calloc(1, sizeof(oh_char *));
  if (opts->opts == OH_NULL)
    goto cleanupret;
  opts->nopts++;
  /* 1 is for the null char */
  opts->opts[0] = (oh_char *)oh_c_malloc(argstrlen + 1);
  if (opts->opts[0] == OH_NULL)
    goto cleanupret;
  oh_c_memcpy(opts->opts[0], argv[0], argstrlen);
  opts->opts[0][argstrlen] = '\0';
  opts->next = OH_NULL;
  if (argsformat[0] == ':')
    return opts;

  for (tailopts = opts, bstr = argsformat;;)
  {
    newopts = OH_NULL;
    estr = oh_c_strchr(bstr, ':');
    OH_ASSERT(estr != OH_NULL);
    argstrlen = estr - bstr - 1; /* 1 is for the '*', '?', '.' */

    for (i = *argcp - 1; i > 0; i--) /* not scan the first argument */
    {
      if (oh_c_strncmp(bstr, argv[i], argstrlen) == 0 &&
        oh_c_strlen(argv[i]) == argstrlen)
      {
        newopts = (oh_as_opts_t *)oh_c_calloc(1, sizeof(oh_as_opts_t));
        if (newopts == OH_NULL)
          goto cleanupret;
        switch (estr[-1])
        {
          case '*': /* comma seperated arguments */
            if (i + 1 >= *argcp)
              goto cleanupret;
            {
              oh_int optcount;
              oh_char *bopt, *eopt;

              for (optcount = 0, bopt = argv[i + 1];; bopt++)
              {
                if (*bopt == ',')
                {
                  optcount++;
                  if (optcount < 0) /* overflow */
                    goto cleanupret;
                }
                else if (*bopt == '\0')
                {
                  optcount++;
                  if (optcount < 0) /* overflow */
                    goto cleanupret;
                  break;
                }
              }

              if (optcount == 0)
                goto cleanupret;

              /* for the first basic argument */
              optcount++;

              newopts->opts =
                (oh_char **)oh_c_calloc(optcount, sizeof(oh_char *));
              if (newopts->opts == OH_NULL)
                goto cleanupret;
              newopts->nopts = optcount;

              newopts->opts[0] = (oh_char *)oh_c_malloc(argstrlen + 1);
              if (newopts->opts[0] == OH_NULL)
                goto cleanupret;
              oh_c_memcpy(newopts->opts[0], argv[i], argstrlen);
              newopts->opts[0][argstrlen] = '\0';

              for (j = 1, bopt = argv[i + 1]; j < optcount; j++)
              {
                if (bopt[0] == ',') /* blank argument */
                {
                  eopt++;
                  newopts->opts[j] = (oh_char *)oh_c_malloc(1);
                  if (newopts->opts[j] == OH_NULL)
                    goto cleanupret;
                  newopts->opts[j][0] = '\0';
                }
                else
                {
                  eopt = oh_c_strchr(bopt, ',');
                  if (eopt == OH_NULL) /* last argument */
                    eopt = bopt + oh_c_strlen(bopt);
                  optstrlen = eopt - bopt;
                  newopts->opts[j] = (oh_char *)oh_c_malloc(optstrlen + 1);
                  if (newopts->opts[j] == OH_NULL)
                    goto cleanupret;
                  oh_c_memcpy(newopts->opts[j], bopt, optstrlen);
                  newopts->opts[j][optstrlen] = '\0';
                }
                bopt = eopt + 1;
              }

              for (j = i + 2; j <= *argcp; j++)
                argv[j - 2] = argv[j];

              (*argcp) -= 2;

              /* remove redundant opts */
              for (j = *argcp - 1; j > 0; j--)
              {
                if (oh_c_strncmp(bstr, argv[j], argstrlen) == 0 &&
                  oh_c_strlen(argv[j]) == argstrlen)
                {
                  for (k = j + 2; k <= *argcp; k++)
                    argv[k - 2] = argv[k];

                  (*argcp) -= 2;
                  j -= 2;
                }
              }
            }

            goto nextargs;

            break;
          case '?': /* one argument */
            if (i + 1 >= *argcp)
              goto cleanupret;
            newopts->opts = (oh_char **)oh_c_calloc(2, sizeof(oh_char *));
            if (newopts->opts == OH_NULL)
              goto cleanupret;
            newopts->nopts = 2;
            newopts->opts[0] = (oh_char *)oh_c_malloc(argstrlen + 1);
            if (newopts->opts[0] == OH_NULL)
              goto cleanupret;
            oh_c_memcpy(newopts->opts[0], argv[i], argstrlen);
            newopts->opts[0][argstrlen] = '\0';
            argstrlen = oh_c_strlen(argv[i + 1]);
            newopts->opts[1] = (oh_char *)oh_c_malloc(argstrlen + 1);
            if (newopts->opts[1] == OH_NULL)
              goto cleanupret;
            oh_c_memcpy(newopts->opts[1], argv[i + 1], argstrlen);
            newopts->opts[1][argstrlen] = '\0';

            for (j = i + 2; j <= *argcp; j++)
              argv[j - 2] = argv[j];

            (*argcp) -= 2;

            /* remove redundant opts */
            for (j = *argcp - 1; j > 0; j--)
            {
              if (oh_c_strncmp(bstr, argv[j], argstrlen) == 0 &&
                oh_c_strlen(argv[j]) == argstrlen)
              {
                for (k = j + 2; k <= *argcp; k++)
                  argv[k - 2] = argv[k];

                (*argcp) -= 2;
                j -= 2;
              }
            }

            goto nextargs;

            break;
          case '.': /* no argument */
            newopts->opts = (oh_char **)oh_c_calloc(1, sizeof(oh_char *));
            if (newopts->opts == OH_NULL)
              goto cleanupret;
            newopts->nopts++;
            newopts->opts[0] = (oh_char *)oh_c_malloc(argstrlen + 1);
            if (newopts->opts[0] == OH_NULL)
              goto cleanupret;
            oh_c_memcpy(newopts->opts[0], argv[i], argstrlen);
            newopts->opts[0][argstrlen] = '\0';

            for (j = i + 1; j <= *argcp; j++)
              argv[j - 1] = argv[j];

            (*argcp)--;

            /* remove redundant opts */
            for (j = *argcp - 1; j > 0; j--)
            {
              if (oh_c_strncmp(bstr, argv[j], argstrlen) == 0 &&
                oh_c_strlen(argv[j]) == argstrlen)
              {
                for (k = j + 1; k <= *argcp; k++)
                  argv[k - 1] = argv[k];

                (*argcp)--;
                j--;
              }
            }

            goto nextargs;

            break;
          default:
            OH_ASSERT(OH_FALSE);
            break;
        }
      }
    }

nextargs:

    if (newopts != OH_NULL)
    {
      tailopts->next = newopts;
      tailopts = newopts;
      tailopts->next = OH_NULL;
    }

    bstr = estr + 1;

    if (bstr[0] == '\0')
      break;
  }

  return opts;

cleanupret:
  oh_as_freeOpts(opts);
  return OH_NULL;
}

/**
 * Deallocates the returned buffer from the oh_as_parseArgs() function.
 *
 * The oh_as_parseArgs() function returns the dynamically allocated memory and
 * that must be deallocated by calling the oh_as_freeOpts() function.
 *
 * @param opts The linked list opts that will be deallocated.
 * @return none.
 */
oh_void oh_as_freeOpts(oh_as_opts_t *opts)
{
  oh_as_opts_t *prevopts;
  oh_int i;

  prevopts = opts;

  while (prevopts)
  {
    opts = opts->next;
    for (i = prevopts->nopts - 1; i >= 0; i--)
      oh_c_free(prevopts->opts[i]);
    oh_c_free(prevopts->opts);
    oh_c_free(prevopts);
    prevopts = opts;
  }
}

/**
 * Finds the specified argument.
 *
 * It searches the "opt" argument in the "opts" parsed arguments linked list.
 *
 * @param opts The parsed arguments that is the data for the search.
 * @param opt The argument that is the key for the search.
 * @return The found parsed argument if successful, OH_NULL otherwise.
 */
oh_as_opts_t *oh_as_findOpt(oh_as_opts_t *opts, const oh_char *opt)
{
  OH_ASSERT(opt != OH_NULL);
  OH_ASSERT(opt[0] == '-');

  for (; opts; opts = opts->next)
    if (oh_c_strcmp(opt, *opts->opts) == 0)
      return opts;

  return OH_NULL;
}

/**
 * Creates the new parsed argument.
 *
 * It creates the new parsed argument. The length of the "opts" array is
 * "nopts". The elements of "opts" are "opt" variable arguments.
 *
 * The returned parsed argument is dynamically allocated, therefore it must be
 * deallocated by calling the oh_as_freeOpts() function.
 *
 * @param nopts The number of "opt"s.
 * @param opt The elements of "opts".
 * @return The newly created parsed argument if successful, OH_NULL otherwise.
 */
oh_as_opts_t *oh_as_newOpt(oh_int nopts, oh_char *opt, ...)
{
  oh_as_opts_t *newopt = OH_NULL;
  oh_int i;
  va_list args;

  OH_ASSERT(nopts > 0);
  OH_ASSERT(opt != OH_NULL);

  newopt = (oh_as_opts_t *)oh_c_calloc(1, sizeof(oh_as_opts_t));
  if (newopt == OH_NULL)
    goto cleanupret;

  newopt->opts = (oh_char **)oh_c_calloc(nopts, sizeof(oh_char *));
  if (newopt->opts == OH_NULL)
    goto cleanupret;
  newopt->nopts = nopts;

  i = 0;
  newopt->opts[i] = oh_c_strdup(opt);
  if (newopt->opts[i] == OH_NULL)
    goto cleanupret;
  i++;

  va_start(args, opt);

  for (; i < nopts; i++)
  {
    oh_char *newarg;
    newarg = va_arg(args, oh_char *);
    newopt->opts[i] = oh_c_strdup(newarg);
    if (newopt->opts[i] == OH_NULL)
    {
      va_end(args);
      goto cleanupret;
    }
  }

  va_end(args);

  return newopt;

cleanupret:
  oh_as_freeOpts(newopt);

  return OH_NULL;
}

/**
 * Adds a parsed argument to the parsed arguments.
 *
 * It adds "opt" parsed argument to the "opts" parsed arguments. If "opts" has
 * already "opt", then remove the previous one and add the new one. It doesn't
 * copy "opt" memory.
 *
 * @param opts The parsed arguments.
 * @param opt The parsed argument that will be added.
 * @return always 0.
 */
oh_int oh_as_addOpt(oh_as_opts_t *opts, oh_as_opts_t *opt)
{
  oh_as_opts_t *prevopt;

  OH_ASSERT(opts != OH_NULL);
  OH_ASSERT(opt != OH_NULL);
  OH_ASSERT(opt->next == OH_NULL);

  for (; opts; prevopt = opts, opts = opts->next)
    if (oh_c_strcmp(*opt->opts, *opts->opts) == 0)
    {
      prevopt->next = opt;
      opt->next = opts->next;
      opts->next = OH_NULL;
      oh_as_freeOpts(opts);
      return 0;
    }

  prevopt->next = opt;

  return 0;
}

/**
 * Removes a parsed argument from the parsed arguments.
 *
 * It removes "opt" parsed argument from the "opts" parsed arguments.
 *
 * @param opts The parsed arguments.
 * @param opt The parsed argument that will be removed.
 * @return 0 if successfully "opt" is removed, not 0 otherwise.
 */
oh_int oh_as_removeOpt(oh_as_opts_t *opts, const oh_char *opt)
{
  oh_as_opts_t *prevopt;

  OH_ASSERT(opts != OH_NULL);
  OH_ASSERT(opt != OH_NULL);
  OH_ASSERT(opt[0] == '-');

  for (; opts; prevopt = opts, opts = opts->next)
    if (oh_c_strcmp(opt, *opts->opts) == 0)
    {
      prevopt->next = opts->next;
      opts->next = OH_NULL;
      oh_as_freeOpts(opts);
      return 0;
    }

  return -1;
}

/**
 * Saves the parsed arguments to the file.
 *
 * It saves the "opts" parsed arguments to the "cfgfile" configuration file.
 *
 * @param opts The parsed arguments to be saved.
 * @param cfgfile The file name to be written.
 * @return 0 if successfully save, not 0 otherwise.
 */
oh_int oh_as_saveToCFG(oh_as_opts_t *opts, const oh_char *cfgfile)
{
  oh_int fd;
  oh_int ret = -1;
  oh_ulong len;
  static const oh_char newline = '\n';
  static const oh_char comma = ',';
  static const oh_char space = ' ';
  static const oh_char quote = '"';

  OH_ASSERT(opts != OH_NULL);
  OH_ASSERT(opts->nopts == 1);
  OH_ASSERT(opts->opts != OH_NULL);
  OH_ASSERT(opts->opts[1] == OH_NULL);
  OH_ASSERT(cfgfile != OH_NULL);

  fd = oh_fs_open(cfgfile, OH_FS_O_WRONLY | OH_FS_O_CREAT | OH_FS_O_TRUNC);
  if (fd < 0)
    goto cleanupret;

  len = oh_c_strlen(opts->opts[0]);
  if (oh_fs_write(fd, &quote, 1) != 1)
    goto cleanupret;
  if (oh_fs_write(fd, opts->opts[0], len) != (oh_long)len)
    goto cleanupret;
  if (oh_fs_write(fd, &quote, 1) != 1)
    goto cleanupret;
  opts = opts->next;
  if (oh_fs_write(fd, &newline, 1) != 1)
    goto cleanupret;

  for (; opts; opts = opts->next)
  {
    oh_int nopts;
    oh_int i;

    nopts = opts->nopts;
    for (i = 0; i < nopts; i++)
    {
      len = oh_c_strlen(opts->opts[i]);
      if (oh_fs_write(fd, opts->opts[i], len) != (oh_long)len)
        goto cleanupret;

      if (i == 0)
      {
        if (oh_fs_write(fd, &space, 1) != 1)
          goto cleanupret;
        if (nopts > 1)
        {
          if (oh_fs_write(fd, &quote, 1) != 1)
            goto cleanupret;
        }
      }
      else if (i != nopts - 1)
      {
        if (oh_fs_write(fd, &comma, 1) != 1)
          goto cleanupret;
      }
      else
      {
        if (nopts > 1)
        {
          if (oh_fs_write(fd, &quote, 1) != 1)
            goto cleanupret;
        }
        if (oh_fs_write(fd, &newline, 1) != 1)
          goto cleanupret;
      }
    }
  }

  oh_fs_close(fd);
  return 0;

cleanupret:
  oh_fs_close(fd);
  return -1;
}

/**
 * Opens the configuration file and parses the file.
 *
 * It opens "cfgfile" configuration file, parses the file with the "argsformat"
 * format, and returns the parsed arguments.
 *
 * The parsed arguments are dynamically allocated. The returned parsed
 * arguments must be deallocated by calling oh_as_freeOpts() function.
 *
 * @param argsformat The parsing format.
 * @param cfgfile The file name to be parsed.
 * @return The parsed arguments if successfully parsed, OH_NULL otherwise.
 */
oh_as_opts_t *oh_as_parseCFG(const oh_char *argsformat, const oh_char *cfgfile)
{
  oh_int fd;
  oh_char *cfgbuf = OH_NULL;
  oh_fs_stat_t stat;
  oh_as_opts_t *opts = OH_NULL;
  oh_int argc;
  oh_char **argv = OH_NULL;

  OH_ASSERT(argsformat != OH_NULL);
  OH_ASSERT(cfgfile != OH_NULL);

  fd = oh_fs_open(cfgfile, OH_FS_O_RDONLY);
  if (fd < 0)
    goto cleanupret;

  if (oh_fs_fstat(fd, &stat) < 0)
    goto cleanupret;

  cfgbuf = (oh_char *)oh_c_malloc(stat.st_size);

  if (cfgbuf == OH_NULL)
    goto cleanupret;

  if (oh_fs_read(fd, cfgbuf, stat.st_size) != (oh_long)stat.st_size)
    goto cleanupret;

  argv = oh_as_parseRawArgs(cfgbuf, &argc);
  if (argv == OH_NULL)
    goto cleanupret;

  opts = oh_as_parseArgs(argsformat, argv, &argc);
  if (opts == OH_NULL)
    goto cleanupret;

cleanupret:
  oh_as_freeArgs(argv);
  oh_c_free(cfgbuf);
  oh_fs_close(fd);

  return opts;
}

/**
 * Clones the parsed arguments.
 *
 * It copies the "opts" and returns the exact copy of the "opts".
 *
 * @param opts The parsed arguments to be cloned.
 * @return The cloned parsed arguments if successful, OH_NULL otherwise.
 */
oh_as_opts_t *oh_as_cloneOpts(oh_as_opts_t *opts)
{
  oh_as_opts_t *tailopts, *newopts, *retopts;
  oh_int num;

  OH_ASSERT(opts != OH_NULL);

  retopts = (oh_as_opts_t *)oh_c_calloc(1, sizeof(oh_as_opts_t));
  if (retopts == OH_NULL)
    return OH_NULL;
  num = opts->nopts;
  retopts->opts = (oh_char **)oh_c_calloc(num, sizeof(oh_char *));
  if (retopts->opts == OH_NULL)
    goto cleanupret;
  retopts->nopts = num;
  for (num--; num >= 0; num--)
  {
    retopts->opts[num] = oh_c_strdup(opts->opts[num]);
    if (retopts->opts[num] == OH_NULL)
      goto cleanupret;
  }
  opts = opts->next;
  tailopts = retopts;

  for (; opts; opts = opts->next)
  {
    newopts = (oh_as_opts_t *)oh_c_calloc(1, sizeof(oh_as_opts_t));
    if (newopts == OH_NULL)
      goto cleanupret;
    tailopts->next = newopts;
    tailopts = newopts;
    num = opts->nopts;
    newopts->opts = (oh_char **)oh_c_calloc(num, sizeof(oh_char *));
    if (newopts->opts == OH_NULL)
      goto cleanupret;
    newopts->nopts = num;
    for (num--; num >= 0; num--)
    {
      newopts->opts[num] = oh_c_strdup(opts->opts[num]);
      if (newopts->opts[num] == OH_NULL)
        goto cleanupret;
    }
  }

  return retopts;

cleanupret:
  oh_as_freeOpts(retopts);
  return OH_NULL;
}

/**
 * Merges the two parsed arguments.
 *
 * It merges the "optsa" and the "optsb" parsed arguments and deallocates the
 * "optsb" parsed arguments.
 *
 * @param optsa The first parsed arguments to be merged.
 * @param optsb The second parsed arguments to be merged.
 * @return The merged parsed arguments if successful, OH_NULL otherwise.
 */
oh_as_opts_t *oh_as_mergeOpts(oh_as_opts_t *optsa, oh_as_opts_t *optsb)
{
  oh_as_opts_t *toptsa, *toptsb, *prevoptsa, *tailoptsb;

  OH_ASSERT(optsa != OH_NULL);
  OH_ASSERT(optsb != OH_NULL);
  OH_ASSERT(oh_c_strcmp(optsa->opts[0], optsb->opts[0]) == 0);
  OH_ASSERT(optsa->nopts == optsb->nopts);

  for (toptsb = optsb; toptsb; toptsb = toptsb->next)
  {
    tailoptsb = toptsb;
  }

  for (toptsa = optsa; toptsa;)
  {
    for (toptsb = optsb; toptsb; toptsb = toptsb->next)
    {
      if (oh_c_strcmp(toptsa->opts[0], toptsb->opts[0]) == 0)
      {
        prevoptsa = toptsa;
        toptsa = toptsa->next;
        continue;
      }
    }
    prevoptsa->next = toptsa->next;
    tailoptsb->next = toptsa;
    tailoptsb = toptsa;
    tailoptsb->next = OH_NULL;
    toptsa = toptsa->next;
  }
  oh_as_freeOpts(optsa);

  return optsb;
}

/**
 * Sets the parsed arguments globally.
 *
 * The global parsed arguments is given by calling oh_as_getGlobalOpts()
 * function.
 *
 * @param opts The parsed arguments that will be set globally.
 * @return none.
 */
oh_void oh_as_setGlobalOpts(oh_as_opts_t *opts)
{
  _oh_as_globalOpts = opts;
}

/**
 * Gets the parsed arguments globally.
 *
 * The global parsed arguments is set by calling oh_as_setGlobalOpts()
 * function.
 *
 * @return The global parsed arguments.
 */
oh_as_opts_t *oh_as_getGlobalOpts(oh_void)
{
  return _oh_as_globalOpts;
}
